@@ -13,6 +13,7 @@
 #include <stdlib.h>
 #include <unistd.h>
 
+#include "chan_capi_platform.h"
 #include "chan_capi20.h"
 #include "chan_capi.h"
 #include "chan_capi_supplementary.h"
@@ -194,7 +195,7 @@ static struct ccbsnr_s *get_ccbsnr_link(char type, unsigned int plci,
  */
 static int ccbsnr_tell_activated(void *data)
 {
-	unsigned int handle = (unsigned int)data;
+	unsigned int handle = (unsigned int)(unsigned long)data;
 	int ret = 0;
 	unsigned int state;
 
@@ -368,7 +369,11 @@ static void	ccbsnr_remote_user_free(_cmsg *CMSG, char type, unsigned int PLCI, _
 #ifdef CC_AST_HAS_EXT_CHAN_ALLOC
 	c = ast_channel_alloc(0, state, handlename, NULL,
 #ifdef CC_AST_HAS_EXT2_CHAN_ALLOC
-		0, ccbsnr->exten, ccbsnr->context, 0,
+		0, ccbsnr->exten, ccbsnr->context,
+#ifdef CC_AST_HAS_LINKEDID_CHAN_ALLOC
+		NULL,
+#endif
+		0,
 #endif
 		"CCBSNR/%x", ccbsnr->handle);
 #else
@@ -832,7 +837,7 @@ int pbx_capi_ccbs(struct ast_channel *c, char *data)
 		for (a = 0; a < 7; a++) {
 		/* Wait for CCBS request indication */
 			if (ast_safe_sleep_conditional(c, 500, ccbsnr_tell_activated,
-			   (void *)handle) != 0) {
+			   (void *)(unsigned long)handle) != 0) {
 				/* we got a hangup */
 				cc_verbose(3, 1,
 					VERBOSE_PREFIX_3 CC_MESSAGE_NAME " ccbs: hangup.\n");
